package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.module.base.pojo.dto.SendMsgDTO;
import com.zanxiang.game.module.base.pojo.params.SendMsgTaskParam;
import com.zanxiang.game.module.base.pojo.vo.SendMsgVO;
import com.zanxiang.game.module.base.util.PageUtil;
import com.zanxiang.game.module.manage.enums.CpSendMsgTaskStatusEnum;
import com.zanxiang.game.module.manage.enums.CpSendMsgTaskTypeEnum;
import com.zanxiang.game.module.manage.service.ICPSendMsgTaskService;
import com.zanxiang.game.module.manage.service.ICpSendMsgLogService;
import com.zanxiang.game.module.mybatis.entity.CpSendMsgTask;
import com.zanxiang.game.module.mybatis.mapper.CpSendMsgTaskMapper;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@Slf4j
public class CPSendMsgTaskServiceImpl extends ServiceImpl<CpSendMsgTaskMapper, CpSendMsgTask> implements ICPSendMsgTaskService {

    @Lazy
    @Resource
    private ICPSendMsgTaskService sendMsgTaskService;

    @Resource
    private ICpSendMsgLogService cpSendMsgLogService;

    @Override
    public ResultVO<Boolean> sendMsg(SendMsgDTO sendMsgDto) {
        try {
            //创建任务
            CpSendMsgTask cpSendMsgTask = createSendMsgTask(sendMsgDto);
            //保存任务
            sendMsgTaskService.save(cpSendMsgTask);
            //发消息
            cpSendMsgLogService.cpSendMsg(cpSendMsgTask.getId(), sendMsgDto.getGameId(), sendMsgDto.getMsg(), sendMsgDto.getRoles());
            //成功后修改状态
            cpSendMsgTask.setStatus(CpSendMsgTaskStatusEnum.SUCCESS_SEND.getValue());
            sendMsgTaskService.updateById(cpSendMsgTask);
            log.info("发送消息成功");
            return ResultVO.ok();
        } catch (Exception e) {
            log.error("发送消息异常, sendMsgDto : {}, e : {}", sendMsgDto, e.getMessage());
            return ResultVO.fail("发送消息异常");
        }
    }

    @Override
    public PageUtil<SendMsgVO> getSendMsgTaskList(SendMsgTaskParam param) {
        Page<CpSendMsgTask> page = sendMsgTaskService.page(new Page<>(param.getPageNum(), param.getPageSize()),
                new LambdaQueryWrapper<CpSendMsgTask>()
                        .eq(param.getGameId() != null, CpSendMsgTask::getGameId, param.getGameId())
                        .eq(StringUtils.isNotEmpty(param.getTaskStatus()), CpSendMsgTask::getStatus, param.getTaskStatus())
                        .orderByDesc(CpSendMsgTask::getCreateTime));
        List<CpSendMsgTask> cpSendMsgTasks = page.getRecords();
        List<SendMsgVO> sendMsgVos = toVo(cpSendMsgTasks);
        return new PageUtil<>(sendMsgVos, page.getTotal(), param.getPageSize(), param.getPageNum(), 0);
    }

    private List<SendMsgVO> toVo(List<CpSendMsgTask> cpSendMsgTasks) {
        if (Objects.isNull(cpSendMsgTasks) || cpSendMsgTasks.isEmpty()) {
            return null;
        }
        return cpSendMsgTasks.stream()
                .map(task -> BeanUtil.copy(task, SendMsgVO.class))
                .collect(Collectors.toList());
    }

    private CpSendMsgTask createSendMsgTask(SendMsgDTO dto) {
        CpSendMsgTask cpSendMsgTask = new CpSendMsgTask();
        cpSendMsgTask.setMsg(dto.getMsg());
        cpSendMsgTask.setRoleIdCount((long) dto.getRoles().size());
        cpSendMsgTask.setTaskName(dto.getTaskName());
        cpSendMsgTask.setTaskCondition(dto.getSendConditionJson());
        cpSendMsgTask.setGameId(dto.getGameId());
        //任务类型
        cpSendMsgTask.setType(CpSendMsgTaskTypeEnum.IMMEDIATE_TASK.getValue());
        //任务状态
        cpSendMsgTask.setStatus(CpSendMsgTaskStatusEnum.WAIT_SEND.getValue());
        cpSendMsgTask.setCreateBy(dto.getCreateBy());
        cpSendMsgTask.setCreateTime(LocalDateTime.now());
        cpSendMsgTask.setUpdateTime(LocalDateTime.now());
        return cpSendMsgTask;
    }
}
